from typing import Any

import pandas as pd


def excel_to_csv(
    excel_path, csv_path, sheet_name=0, field_separator="|"
) -> Any:
    """
    Converts an Excel sheet into a delimited CSV file.

    Parameters:
        excel_path (str or Path): Path to the input Excel file.
        csv_path (str or Path): Destination path for the output CSV file.
        sheet_name (int or str, optional): Sheet index or name to read from.
            Default is the first sheet (0).
        field_separator (str, optional): Delimiter to use in the CSV.
            Default is '|'.

    Returns:
        Any: The path to the generated CSV file.
    """

    df: pd.DataFrame = pd.read_excel(excel_path, sheet_name=sheet_name)
    df.to_csv(csv_path, sep=field_separator, index=False)
    return csv_path


def csv_to_excel(
    csv_path, excel_path, sheet_name="cmor_table", field_separator="|"
) -> str:
    """
    Converts a delimited CSV file into an Excel sheet.

    Parameters:
        csv_path (str or Path): Path to the input CSV file.
        excel_path (str or Path): Destination path for the output Excel file.
        sheet_name (str, optional): Name of the sheet in the Excel file.
            Default is 'cmor_table'.
        field_separator (str, optional): Delimiter used in the CSV.
            Default is '|'.

    Returns:
        str: The path to the generated Excel file.
    """

    df: pd.DataFrame = pd.read_csv(csv_path, sep=field_separator)
    df.to_excel(excel_path, sheet_name=sheet_name, index=False)
    return excel_path
